<?php

namespace Drupal\search_api_lunr;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\search_api\Item\Field;
use Drupal\search_api\Item\FieldInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The PageAttachments class.
 */
class PageAttachments implements ContainerInjectionInterface {

  /**
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $serverStorage;

  /**
   * Create and instance of PageAttachments.
   */
  public function __construct($serverStorage) {
    $this->serverStorage = $serverStorage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')->getStorage('search_api_server')
    );
  }

  /**
   * Implements hook_page_attachments().
   */
  public function attachments(&$page) {
    $settings = [
      'servers' => [],
    ];
    /** @var \Drupal\search_api\ServerInterface[] $servers */
    /** @var \Drupal\search_api_lunr\Plugin\search_api\backend\SearchApiJsBackend $backend */
    $servers = $this->serverStorage->loadByProperties([
      'backend' => 'search_api_lunr',
    ]);
    foreach ($servers as $server) {
      foreach ($server->getIndexes() as $index) {
        $backend = $server->getBackend();
        $settings['servers'][$server->id()]['indexes'][$index->id()]['fileList'] = $backend->getIndexFileList($index);
        $settings['servers'][$server->id()]['indexes'][$index->id()]['fields'] = array_map([$this, 'processField'], $index->getFields());
        $settings['servers'][$server->id()]['indexes'][$index->id()]['version'] = $backend->getIndexVersion($index);
      }
    }
    $page['#attached']['drupalSettings']['search_api_lunr'] = $settings;
    $page['#attached']['library'][] = 'search_api_lunr/api';
  }

  /**
   * Process a search API field into settings.
   *
   * @param \Drupal\search_api\Item\FieldInterface $field
   *   The search API field.
   *
   * @return array
   *   An array of configuration.
   */
  public function processField(FieldInterface $field) {
    $build = [
      'boost' => $field->getBoost(),
    ];
    if (strpos($field->getDataTypePlugin()->getPluginId(), 'search_api_lunr_') === 0) {
      $build['special_type'] = substr($field->getDataTypePlugin()->getPluginId(), strlen('search_api_lunr_'));
    }
    return $build;
  }

}
